#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include "robot.h"
#include "scene.h"

#define MIN_RADIUS	6

void display(void);
void reshape (int w, int h);
void mouse (int button, int state, int x, int y); 
void motion (int x, int y);
void keyb (unsigned char key, int x, int y);
void idle(void);

static int prev_x = -1, prev_y = -1;
static double phi = 25;
static double theta = 0; /*spherical coordinates*/
static double distance = 10;

double* offset_array; // offset array
int num_robots = 1;
double max_rad;

Node* root;

int main (int argc, char** argv) {
	glutInitWindowSize(800, 600);
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutCreateWindow("Bob the Robot! :-)P");

	glEnable(GL_CULL_FACE);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	glEnable(GL_NORMALIZE);

	if (argc > 1 && isdigit(argv[1][0])) {
		num_robots = atoi(argv[1]);
	}

	/*offset_array = malloc(num_robots * sizeof(double));*/
	offset_array = new double[num_robots];
	for (int i=0; i< num_robots; i++) {
		offset_array[i] = ((double)rand() / (double)RAND_MAX) * 15.0;
	//	offset_array[i] = 0;
	}

	max_rad = (double)num_robots * 4.0 + MIN_RADIUS * 2;
	if(max_rad < 20.0) {
		max_rad = 20.0;
	}

	root = make_robot();
	if (!(create_scene(max_rad))) {
		fprintf(stderr, "failed to load scene!\n");
	}

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyb);
	glutMouseFunc(mouse);
	glutMotionFunc(motion);
	glutIdleFunc(idle);
	glutMainLoop();
	return 0;
}


void display (void) {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); //clear buffers
	glMatrixMode(GL_MODELVIEW); 	// which matrix we want to change
	glLoadIdentity();				// model view matrix = I
	/*
	 	other way:
		gluLookAt(0, 0, 10, 0, 0, 0, 0, 1, 0);
		(position, target, up vector)
	 */

	glTranslatef(0, 0, -distance);		/* set the camera to (0, 0, 10) => move 
									   everything 10 units along the z axis*/

	glRotatef(phi, 1, 0, 0);
	glRotatef(theta, 0, 1, 0);

	float light_pos[4] = {-1.0, 1.0, 2.0, 0.0};
	float light_diffuse[3] = {0.8, 0.8, 0.8};
	glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light_diffuse);

	float light_pos2[4] = {0, 20, 0, 1};
	float light_diffuse2[3] = {0.55, 0.55, 0.55};
	glLightfv(GL_LIGHT1, GL_POSITION, light_pos2);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, light_diffuse2);
	glLightfv(GL_LIGHT1, GL_SPECULAR, light_diffuse2);

	double t = (double)glutGet(GLUT_ELAPSED_TIME)/1000;

	for (int i=0; i<num_robots; i++) {
		animate(t + offset_array[i], root, i * 4 + MIN_RADIUS);
		root->draw();
	}

	draw_scene();
	glutSwapBuffers();				/* we use double buffering => after drawing
									   swap buffers and render buffer content*/
}

void reshape (int w, int h) {
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45, (double)w/h, 1, 10000);
}

static int bn;
void mouse (int button, int state, int x, int y) {
	bn = button;
	prev_x = x;
	prev_y = y;
}

void motion (int x, int y) {
	switch (bn) {
	case GLUT_LEFT_BUTTON:
		theta += x - prev_x;
		phi += y - prev_y;

		theta = theta < 0 ? 360 + theta : theta;
		theta = theta > 360 ? theta - 360 : theta;

		phi = phi < -90 ? 90 + phi : phi;
		phi = phi > 90 ? phi - 90 : phi;
		break;

	case GLUT_RIGHT_BUTTON:
		distance *= (y - prev_y) * 0.01 + 1;
		if(distance < 0.0) {
			distance = 0.0;
		}
		break;
	}

	prev_x = x;
	prev_y = y;
}

void keyb(unsigned char key, int x, int y) {
	switch(key) {
	case 27:
		exit(0);
	}

}

void idle(void) {
	glutPostRedisplay();
}
