#ifdef __APPLE__
#include <OpenGL/gl.h>
#else
#include <GL/gl.h>
#endif

#include "node.h"

Node::Node() {
	parent = 0;
	rpart = 0;
	name = 0;
	axis = Vector3(1.0, 0.0, 0.0);
	angle = 0;
}

Node::Node(RPart* rpart, int name) {
	parent = 0;
	this->rpart = rpart;
	this->name = name;
	axis = Vector3(1.0, 0.0, 0.0);
	angle = 0;
}

Node::~Node() {
	for (int i=0; i<(int)children.size(); i++) {
		delete children[i];
	}
	delete rpart;
}

void Node::add_child(Node* node) {
	children.push_back(node);
	node->parent = this;
}

void Node::set_parent(Node* node) {
	parent = node;
}

RPart* Node::get_part() const {
	return rpart;
}

void Node::set_angle(double angle) {
	this->angle = angle;
}

void Node::set_position(const Vector3 &position) {
	this->position = position;
}

void Node::set_axis(const Vector3 &axis) {
	this->axis = axis;
}

Node* Node::get_child(int idx) const {
	return children[idx];
}

int Node::num_children() const {
	return (int) children.size();
}

void Node::draw() const {
	glPushMatrix();
	glTranslatef(position.x, position.y, position.z);
	glRotatef(angle, axis.x, axis.y, axis.z);
	rpart->draw();

	int sz = (int)children.size();
	for (int i=0; i < sz; i++) {
		children[i]->draw();
	}

	glPopMatrix();

}

Node* get_node(int name, Node* node) {
	if (node->name == name) {
		return node;
	}

	Node* tmp;
	
	int sz = (int)node->children.size();
	for (int i=0; i < sz; i++) {
		tmp = get_node(name, node->children[i]);
		if (tmp) {
			return tmp;
		}
	}

	return 0;
}

