#include <math.h>
#include "robot.h"
#include "rpart.h"

enum {BODY, HEAD, HAIR, POSTIS, REYE, RINEYE, RLINE, 
	REYEBROW, REAR, LEYE, LINEYE, LEYEBROW, LEAR, MOUTH, RUARM, RLARM, RHAND, 
	LUARM, LLARM, LHAND, RULEG, RLLEG, RFOOT, LULEG, LLLEG, LFOOT, RFT};

Node* make_robot() {
	/*body*/
	RPart* rpart = new RPart(Vector3(2, 2, 0.85), Vector3(0, 0, 0));
	Node* root = new Node(rpart, BODY);
	root->get_part()->set_color(Vector3(0.34, 0.13, 0.13));

	/*head*/
	rpart = new RPart(Vector3(0.65, 0.85, 0.65), Vector3(0, 0, 0));
	Node* head = new Node(rpart, HEAD);
	root->add_child(head);
	head->set_position(Vector3(0, 1.5, 0));
	head->get_part()->set_color(Vector3(1, 0.56, 0));

	/*left ear*/
	rpart = new RPart(Vector3(0.02, 0.2, 0.15), Vector3(0, 0, 0));
	Node* lear = new Node(rpart, LEAR);
	head->add_child(lear);
	lear->set_position(Vector3(0.325, -0.07, 0.15));
	lear->get_part()->set_color(Vector3(1, 0.56, 0));

	/*right ear*/
	rpart = new RPart(Vector3(0.02, 0.2, 0.15), Vector3(0, 0, 0));
	Node* rear = new Node(rpart, REAR);
	head->add_child(rear);
	rear->set_position(Vector3(-0.325, -0.07, 0.15));
	rear->get_part()->set_color(Vector3(1, 0.56, 0));

	/*hair*/
	rpart = new RPart(Vector3(0.66, 0.2, 0.66), Vector3(0, 0, 0));
	Node* hair = new Node(rpart, HAIR);
	root->add_child(hair);
	hair->set_position(Vector3(0, 1.95, 0));
	hair->get_part()->set_color(Vector3(0, 0, 0));

	/*postis*/
	rpart = new RPart(Vector3(0.66, 0.66, 0.845), Vector3(0, 0, 0));
	Node* postis = new Node(rpart, POSTIS);
	head->add_child(postis);
	postis->set_position(Vector3(0, 0.25, -0.1));
	postis->get_part()->set_color(Vector3(0, 0, 0));

	/*right eye*/
	rpart = new RPart(Vector3(0.1, 0.2, 0.1), Vector3(0, 0, 0));
	Node* reye = new Node(rpart, REYE);
	head->add_child(reye);
	reye->set_position(Vector3(-0.1, 0.1, 0.28));
	reye->get_part()->set_color(Vector3(1, 1, 1));

	/*right eye - black dot*/
	rpart = new RPart(Vector3(0.05, 0.05, 0.11), Vector3(0, 0, 0)); 
	Node* rineye = new Node(rpart, RINEYE);
	reye->add_child(rineye);
	rineye->set_position(Vector3(0, 0, 0));
	rineye->get_part()->set_color(Vector3(0, 0, 0));

	/*right eyebrow*/
	rpart = new RPart(Vector3(0.1, 0.02, 0.1), Vector3(0, 0, 0));
	Node* reyebrow = new Node(rpart, REYEBROW);
	reye->add_child(reyebrow);
	reyebrow->set_position(Vector3(0, 0.15, 0));

	/*left eye*/
	rpart = new RPart(Vector3(0.1, 0.2, 0.1), Vector3(0, 0, 0));
	Node* leye = new Node(rpart, LEYE);
	head->add_child(leye);
	leye->set_position(Vector3(0.1, 0.1, 0.28));
	leye->get_part()->set_color(Vector3(1, 1, 1));

	/*left eye - black dot*/
	rpart = new RPart(Vector3(0.05, 0.05, 0.11), Vector3(0, 0, 0)); 
	Node* lineye = new Node(rpart, LINEYE);
	leye->add_child(rineye);
	lineye->set_position(Vector3(0, 0, 0));
	lineye->get_part()->set_color(Vector3(0, 0, 0));

	/*mouth*/
	rpart = new RPart(Vector3(0.1, 0.03, 0.1), Vector3(0, 0, 0));
	Node* mouth = new Node(rpart, MOUTH);
	head->add_child(mouth);
	mouth->set_position(Vector3(0, -0.18, 0.28));
	mouth->get_part()->set_color(Vector3(0, 0, 0));

	/*left eyebrow*/
	rpart = new RPart(Vector3(0.1, 0.02, 0.1), Vector3(0, 0, 0));
	Node* leyebrow = new Node(rpart, LEYEBROW);
	leye->add_child(leyebrow);
	leyebrow->set_position(Vector3(0, 0.15, 0));
	
	/*
	
	   Hands open :-)P

	*right upper arm*
	rpart = new RPart(Vector3(1, 0.5, 0.5), Vector3(0.5, 0, 0));
	Node* ruarm = new Node(rpart, RUARM);
	root->add_child(ruarm);
	ruarm->set_position(Vector3(-1, 0.75, 0));

	*right lower arm*
	rpart = new RPart(Vector3(1, 0.5, 0.5), Vector3(0.5, 0, 0));
	Node* rlarm = new Node(rpart, RLARM);
	ruarm->add_child(rlarm);
	rlarm->set_position(Vector3(-1, 0, 0));

	*right hand*
	rpart = new RPart(Vector3(0.5, 0.6, 0.5), Vector3(0, 0, 0));
	Node* rhand = new Node(rpart, RHAND);
	rlarm->add_child(rhand);
	rhand->set_position(Vector3(-1.25, 0, 0));

	*left upper arm*
	rpart = new RPart(Vector3(1, 0.5, 0.5), Vector3(-0.5, 0, 0));
	Node* luarm = new Node(rpart, LUARM);
	root->add_child(luarm);
	luarm->set_position(Vector3(1, 0.75, 0));

	*left lower arm*
	rpart = new RPart(Vector3(1, 0.5, 0.5), Vector3(-0.5, 0, 0));
	Node* llarm = new Node(rpart, LLARM);
	luarm->add_child(llarm);
	llarm->set_position(Vector3(1, 0, 0));

	*left hand*
	rpart = new RPart(Vector3(0.5, 0.6, 0.5), Vector3(0, 0, 0));
	Node* lhand = new Node(rpart, LHAND);
	llarm->add_child(lhand);
	lhand->set_position(Vector3(1.25, 0, 0));
	 
	 */

	/*right upper leg*/
	rpart = new RPart(Vector3(0.5, 1, 0.5), Vector3(0, 0.5, 0));
	Node* ruleg = new Node(rpart, RULEG);
	root->add_child(ruleg);
	ruleg->set_position(Vector3(-0.55, -1, 0));
	ruleg->get_part()->set_color(Vector3(0, 0, 0.29));

	/*right lower leg*/
	rpart = new RPart(Vector3(0.5, 1, 0.5), Vector3(0, 0.5, 0));
	Node* rlleg = new Node(rpart, RLLEG);
	ruleg->add_child(rlleg);
	rlleg->set_position(Vector3(0, -1, 0));
	rlleg->get_part()->set_color(Vector3(0, 0, 0.29));

	
	/*left upper leg*/
	rpart = new RPart(Vector3(0.5, 1, 0.5), Vector3(0, 0.5, 0));
	Node* luleg = new Node(rpart, LULEG);
	root->add_child(luleg);
	luleg->set_position(Vector3(0.55, -1, 0));
	luleg->get_part()->set_color(Vector3(0, 0, 0.29));

	/*left lower leg*/
	rpart = new RPart(Vector3(0.5, 1, 0.5), Vector3(0, 0.5, 0));
	Node* llleg = new Node(rpart, LLLEG);
	luleg->add_child(llleg);
	llleg->set_position(Vector3(0, -1, 0));
	llleg->get_part()->set_color(Vector3(0, 0, 0.29));

	/* hands down*/

	/*right upper arm*/
	rpart = new RPart(Vector3(0.5, 1, 0.5), Vector3(0, 0.5, 0));
	Node* ruarm = new Node(rpart, RUARM);
	root->add_child(ruarm);
	ruarm->set_position(Vector3(-1.25, 1, 0));
	ruarm->get_part()->set_color(Vector3(0.34, 0.13, 0.13));

	/*right lower arm*/
	rpart = new RPart(Vector3(0.5, 1, 0.5), Vector3(0, 0.5, 0));
	Node* rlarm = new Node(rpart, RLARM);
	ruarm->add_child(rlarm);
	rlarm->set_position(Vector3(0, -1, 0));
	rlarm->get_part()->set_color(Vector3(0.34, 0.13, 0.13));

	/*right hand*/
	rpart = new RPart(Vector3(0.5, 0.5, 0.5), Vector3(0, 0, 0));
	Node* rhand = new Node(rpart, RHAND);
	rlarm->add_child(rhand);
	rhand->set_position(Vector3(0, -1.25, 0));
	rhand->get_part()->set_color(Vector3(1, 0.56, 0));

	/*left upper arm*/
	rpart = new RPart(Vector3(0.5, 1, 0.5), Vector3(0, 0.5, 0));
	Node* luarm = new Node(rpart, LUARM);
	root->add_child(luarm);
	luarm->set_position(Vector3(1.25, 1, 0));
	luarm->get_part()->set_color(Vector3(0.34, 0.13, 0.13));

	/*left lower arm*/
	rpart = new RPart(Vector3(0.5, 1, 0.5), Vector3(0, 0.5, 0));
	Node* llarm = new Node(rpart, LLARM);
	luarm->add_child(llarm);
	llarm->set_position(Vector3(0, -1, 0));
	llarm->get_part()->set_color(Vector3(0.34, 0.13, 0.13));

	/*left hand*/
	rpart = new RPart(Vector3(0.5, 0.5, 0.5), Vector3(0, 0, 0));
	Node* lhand = new Node(rpart, LHAND);
	llarm->add_child(lhand);
	lhand->set_position(Vector3(0, -1.25, 0));
	lhand->get_part()->set_color(Vector3(1, 0.56, 0));

	/*left foot*/
	rpart = new RPart(Vector3(0.5, 0.3, 0.7), Vector3(0, 0, -0.35));
	Node* lfoot = new Node(rpart, LFOOT);
	llleg->add_child(lfoot);
	lfoot->set_position(Vector3(0, -1.1, -0.35));
	lfoot->get_part()->set_color(Vector3(0.31, 0.2, 0.09));

	/*right foot*/
	rpart = new RPart(Vector3(0.5, 0.3, 0.7), Vector3(0, 0, -0.35));
	Node* rfoot = new Node(rpart, RFOOT);
	rlleg->add_child(rfoot);
	rfoot->set_position(Vector3(0, -1.1, -0.35));
	rfoot->get_part()->set_color(Vector3(0.31, 0.2, 0.09));

	/*right foot extension
	rpart = new RPart(Vector3(0.5, 0.5, 0.7), Vector3(0, 0, 0));
	Node* lft = new Node(rpart, LFT);
	lfoot->add_child(lft);
	lft->set_position(Vector3(0, 0, 0.25));
	lft->get_part()->set_color(Vector3(0.31, 0.2, 0.09));*/
	
	return root;
}

void animate(double t, Node* root, double radius) {

	t *= 4;

	Node* node = get_node(BODY, root);
	node->set_position(Vector3(sin(t/5)*radius, 0, cos(t/5)*radius));
	double angle = t/5 * 180/M_PI + 90;
	node->set_angle(angle);
	node->set_axis(Vector3(0, 1, 0));

	/* leg animation */
	
	node = get_node(RULEG, root);
	node->set_angle(45 * sin(t));
	node->set_axis(Vector3(1, 0, 0));

	node = get_node(RLLEG, root);
	node->set_angle(45 * (sin(t) + 1));
	node->set_axis(Vector3(1, 0, 0));
		
	node = get_node(LULEG, root);
	node->set_angle(-45 * sin(t));
	node->set_axis(Vector3(1, 0, 0));

	node = get_node(LLLEG, root);
	node->set_angle(-45 * (sin(t) -1));
	node->set_axis(Vector3(1, 0, 0));

	/* arm animation */

	node = get_node(RUARM, root);
	node->set_angle(-45 * sin(t));
	node->set_axis(Vector3(1, 0, 0));

	node = get_node(RLARM, root);
	node->set_angle(16 * (sin(t) - 1) - 15);
	node->set_axis(Vector3(1, 0, 0));

	node = get_node(LUARM, root);
	node->set_angle(45 * sin(t));
	node->set_axis(Vector3(1, 0, 0));

	node = get_node(LLARM, root);
	node->set_angle(-16 * (sin(t) + 1) - 15);
	node->set_axis(Vector3(1, 0, 0));
}
