#ifdef __APPLE__
#include <OpenGL/gl.h>
#else
#include <GL/gl.h>
#endif

#include "rpart.h"

static void cube();

RPart::RPart() {
	size = Vector3(1.0, 1.0, 1.0);
	pivot = Vector3(0.0, 0.0, 0.0);
	color = Vector3(0.0, 0.0, 0.0);
}

RPart::RPart(const Vector3 &size, const Vector3 &pivot) {
	this->size = size;
	this->pivot = pivot;
}

void RPart::set_size(const Vector3 &size) {
	this->size = size;
}

void RPart::set_pivot(const Vector3 &pivot) {
	this->pivot = pivot;
}

void RPart::draw() const {
	float rgb[4];

	glPushMatrix();
	glTranslatef(-pivot.x, -pivot.y, -pivot.z);
	glScalef(size.x/2, size.y/2, size.z/2);
	
	rgb[0] = (float)color.x;
	rgb[1] = (float)color.y;
	rgb[2] = (float)color.z;
	rgb[3] = 1;

	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, rgb); 
	cube();
	glPopMatrix();
}

void RPart::set_color(const Vector3 &color) {
	this->color = color;
}

const Vector3 RPart::get_color() {
	return color;
}

static void cube() {
	
//	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

	glBegin(GL_QUADS);

	/*front polygon*/
	glNormal3f(0, 0, 1);
	glVertex3f(1, -1, 1);
	glVertex3f(1, 1, 1);
	glVertex3f(-1, 1, 1);
	glVertex3f(-1, -1, 1);

	/*back polygon*/
	glNormal3f(0, 0, -1);
	glVertex3f(-1, 1, -1);
	glVertex3f(1, 1, -1);
	glVertex3f(1, -1, -1);
	glVertex3f(-1, -1, -1);

	/*right side polygon*/
	glNormal3f(1, 0, 0);
	glVertex3f(1, -1, 1);
	glVertex3f(1, -1, -1);
	glVertex3f(1, 1, -1);
	glVertex3f(1, 1, 1);

	/*left side polygon*/
	glNormal3f(-1, 0, 0);
	glVertex3f(-1, -1, 1);
	glVertex3f(-1, 1, 1);
	glVertex3f(-1, 1, -1);
	glVertex3f(-1, -1, -1);

	/*top polygon*/
	glNormal3f(0, 1, 0);
	glVertex3f(1, 1, 1);
	glVertex3f(1, 1, -1);
	glVertex3f(-1, 1, -1);
	glVertex3f(-1, 1, 1);

	/*bottom polygon*/
	glNormal3f(0, -1, 0);
	glVertex3f(-1, -1, 1);
	glVertex3f(-1, -1, -1);
	glVertex3f(1, -1, -1);
	glVertex3f(1, -1, 1);

	glEnd();
}
