#include <stdio.h>
#include <stdlib.h>

#ifdef __APPLE__
#include <OpenGL/gl.h>
#else
#include <GL/gl.h>
#endif

#include "image.h"
#include "scene.h"

#define TREP (1/4.0)

static double mr;
static unsigned int tex1, tex2;
void static quad(unsigned int tex);

bool create_scene(double max_radius) {
	const char* fname = "data/carpet.ppm";
	const char* fname2 = "data/aerostato.ppm";

	Image *img = new Image();
	Image *img2 = new Image();
	mr = max_radius;

	if (!(img->load(fname))) {
		fprintf(stderr, "failed to load texture: %s\n", fname);
		return false;
	}

	if (!(img2->load(fname2))) {
		fprintf(stderr, "failed to load texture: %s\n", fname2);
		return false;
	}

	/*create opengl texture and set image pixels*/
	glGenTextures(1, &tex1);
	glBindTexture(GL_TEXTURE_2D, tex1);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, 4, img->width, img->height, 0, GL_RGB, GL_UNSIGNED_BYTE, img->get_pixels());

	/*create opengl texture and set image pixels*/
	glGenTextures(1, &tex2);
	glBindTexture(GL_TEXTURE_2D, tex2);	
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, 4, img2->width, img2->height, 0, GL_RGB, GL_UNSIGNED_BYTE, img2->get_pixels());

	delete img;
	delete img2;

	return true;
}

void draw_scene() {
	float white[] = {1, 1, 1, 1};

	glEnable(GL_TEXTURE_2D);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, white);

	/*floor*/
	glPushMatrix();
	glBindTexture(GL_TEXTURE_2D, tex1);
	glScalef(mr, 1, mr);
	glTranslatef(0, -3.3, 0);
	quad(tex1);
	glPopMatrix();

	/*back*/
	glPushMatrix();
	glBindTexture(GL_TEXTURE_2D, tex2);
	glTranslatef(0, -3.3+mr, -mr);
	glScalef(mr, mr, 1);
	glRotatef(90, 1, 0, 0);
	quad(tex2);
	glPopMatrix();

	/*right*/
	glPushMatrix();
	glBindTexture(GL_TEXTURE_2D, tex2);
	glTranslatef(mr, -3.3+mr, 0);
	glScalef(1, mr, mr);
	glRotatef(-90, 0, 1, 0);
	glRotatef(90, 1, 0, 0);
	quad(tex2);
	glPopMatrix();

	/* front */
	glPushMatrix();
	glBindTexture(GL_TEXTURE_2D, tex2);
	glTranslatef(0, -3.3 + mr, mr);
	glScalef(mr, mr, 1);
	glRotatef(180, 0, 1, 0);
	glRotatef(90, 1, 0, 0);
	quad(tex2);
	glPopMatrix();

	/* left */
	glPushMatrix();
	glBindTexture(GL_TEXTURE_2D, tex2);
	glTranslatef(-mr, -3.3+mr, 0);
	glScalef(1, mr, mr);
	glRotatef(90, 0, 1, 0);
	glRotatef(90, 1, 0, 0);
	quad(tex2);
	glPopMatrix();

	glDisable(GL_TEXTURE_2D);
}

void static quad(unsigned int tex) {
	glBegin(GL_QUADS);
	glNormal3f(0, 1, 0);
	glTexCoord2f(0, mr*TREP);	glVertex3f(-1, 0, 1);
	glTexCoord2f(mr*TREP, mr*TREP); glVertex3f(1, 0, 1);
	glTexCoord2f(mr*TREP, 0); glVertex3f(1, 0, -1);
	glTexCoord2f(0, 0); glVertex3f(-1, 0, -1);
	glEnd();
}
